part of serializer_dart;

/**
 * Encoder implementation using reflection (mirrors)
 */
class ReflectionEncoder implements Encoder {
  Map<Symbol, TypeEncoder> _encoders = new Map();
  
  ReflectionEncoder() {
    registerCustomEncoder("DateTime", new DateTimeEncoder());
  }
  
  void registerCustomEncoder(String simpleClassName, TypeEncoder encoder) {
    _encoders[new Symbol(simpleClassName)] = encoder;  
  }
  
  Object encodeObject(Object object) {
    if (_isPrimative(object)) {
      return _serializePrimitive(object);
    }
    else if (object is Map) {
      return _serializeMap(object);
    }
    else if (object is List) {
      return _serializeList(object);
    }
    else {
      mirrors.InstanceMirror instanceMirror = mirrors.reflect(object);
      if (_encoders.containsKey(instanceMirror.type.simpleName)) {
        return _encoders[instanceMirror.type.simpleName].encode(object, this);
      }
      else {
        return _serializeObject(instanceMirror);
      }
    }
  }
  
  bool _isPrimative(Object object){
    if (object is num || object is bool || object is String || object == null) {
      return true;
    } 
    else {
      return false;
    }
  }
  
  Object _serializePrimitive(Object object) {
    return object;
  }
  
  Object _serializeList(Object object) {
    List result = [];
    (object as List).forEach((item){
      result.add(encodeObject(item));
    });
    return result;
  }
  
  Object _serializeMap(Object object) {
    Map result = new Map();
    (object as Map).forEach((key, value){
      result[key] = encodeObject(value);
    });
    return result;
  }
  
  Object _serializeObject(mirrors.InstanceMirror instanceMirror) {
    var classMirror = instanceMirror.type;
    // TODO read metadata to help serialization
    // custom names, skip fields...
    // classMirror.metadata
    var resultMap = new Map();
    
    // for each getter:
    classMirror.getters.forEach((getterKey, getterMirror) {
      if (!getterMirror.isPrivate && !getterMirror.isStatic) {
        mirrors.InstanceMirror field = instanceMirror.getField(getterKey);
        Object reflectee = field.reflectee;
        resultMap[mirrors.MirrorSystem.getName(getterKey)] = encodeObject(reflectee);
      }
    });
     // for each field
    classMirror.variables.forEach((varKey, varMirror) {
      if (!varMirror.isPrivate && !varMirror.isStatic) {
        mirrors.InstanceMirror field = instanceMirror.getField(varKey);
        Object reflectee = field.reflectee;
        resultMap[mirrors.MirrorSystem.getName(varKey)] = encodeObject(reflectee);
      }
    });  
    return resultMap;
  }
}